
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using ClarionSimulation;
using WorldServerLibrary;
using WorldServerLibrary.Model;
using WorldServerLibrary.Exceptions;

namespace ClarionDEMO
{
	class MainClass
	{
		#region properties
		private WorldServer worldServer = null;
        private ClarionAgent agent;
        String creatureId = String.Empty;
        String creatureName = String.Empty;
		double prad = 0;
		private IList<Thing> coisas = null;
		#endregion

		#region constructor
		public MainClass() {
			Console.WriteLine ("Clarion Demo V0.3");
			agent = new ClarionAgent();
            agent.OnNewVisualSensorialInformation += new InputVisualSensorialInformationEventHandler(agent_OnNewVisualSensorialInformation);
            agent.OnNewExternalActionSelected += new OutputActionChunkEventHandler(agent_OnNewExternalActionSelected);
			try
            {
                worldServer = new WorldServer("localhost", 4011);
                String message = worldServer.Connect();

                if (worldServer != null && worldServer.IsConnected)
                {
                    Console.Out.WriteLine ("[SUCCESS] " + message + "\n");
                    worldServer.NewCreature(400, 200, 0, out creatureId, out creatureName);
                    //worldServer.NewBrick(4, 747, 2, 800, 567);
                    //worldServer.NewBrick(4, 50, -4, 747, 47);
                    //worldServer.NewBrick(4, 49, 562, 796, 599);
                    //worldServer.NewBrick(4, -2, 6, 50, 599);

					worldServer.NewFood(0,110,90);
					worldServer.NewFood(1,415,315);
					worldServer.NewJewel(1,550,335);
					worldServer.NewJewel(0,90,335);
					worldServer.NewJewel(2,250,135);
					worldServer.NewJewel(3,350,235);

					String status=worldServer.createLeafLet(creatureName);
					Console.Out.WriteLine("Status: " + status + "\n");

                    if (!String.IsNullOrWhiteSpace(creatureId))
                    {
                        worldServer.SendStartCamera(creatureId);
                        worldServer.SendStartCreature(creatureId);
                    }

                    Console.Out.WriteLine("Creature created with name: " + creatureId + "\n");
                    agent.Run();
					Console.Out.WriteLine("Running Simulation ...\n");
                }
            }
            catch (WorldServerInvalidArgument invalidArtgument)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Invalid Argument: {0}\n", invalidArtgument.Message));
            }
            catch (WorldServerConnectionError serverError)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Is is not possible to connect to server: {0}\n", serverError.Message));
            }
            catch (Exception ex)
            {
                Console.Out.WriteLine(String.Format("[ERROR] Unknown Error: {0}\n", ex.Message));
            }
		}
		#endregion

		#region Methods
		public static void Main (string[] args)	{   
			new MainClass();
		}

        IList<Thing> agent_OnNewVisualSensorialInformation()
        {
            IList<Thing> response = null;

            if (worldServer != null && worldServer.IsConnected)
            {
                response = worldServer.SendGetCreatureState(creatureName);
				prad = (Math.PI / 180) * response.First().Pitch;
			    while (prad > Math.PI) prad -= 2 * Math.PI;
			    while (prad < - Math.PI) prad += 2 * Math.PI;
            }
			coisas=response;
            return response;
        }

		void buscaJewelMaisProxima ()
		{
			Thing menor=null;
			Double disMenor=Double.MaxValue;
			foreach (Thing item in coisas) {
				if((item.CategoryId == Thing.CATEGORY_JEWEL) 
				   && item.DistanceToCreature <= disMenor){
					disMenor=item.DistanceToCreature;
					menor=item;
				}
			}
			if(menor!=null){                    
				worldServer.SendSetGoTo(creatureId,1.0,1.0,menor.X1,menor.Y1);
			}}

        void agent_OnNewExternalActionSelected(ClarionAgentActionType externalAction)
        {
            if (worldServer != null && worldServer.IsConnected)
			{ 	Thing menor=null;
				Double disMenor=Double.MaxValue;
                switch (externalAction)
                {
					case ClarionAgentActionType.TAKEJEWEL:
					menor=null;
					disMenor=Double.MaxValue;
					foreach (Thing item in coisas) {
						if((item.CategoryId == Thing.CATEGORY_JEWEL) 
						   && item.DistanceToCreature <= disMenor){
							disMenor=item.DistanceToCreature;
							menor=item;
						}
					}
					if(menor!=null){
						if(disMenor<=30.0){
							worldServer.SendSackIt(creatureId,menor.Name);
						}
						}   
                        break;
					case ClarionAgentActionType.EATFOOD:
					//Creature
					menor=null;
					disMenor=Double.MaxValue;
					foreach (Thing item in coisas) {
						if((item.CategoryId == Thing.CATEGORY_FOOD || item.CategoryId == Thing.CATEGORY_NPFOOD || item.CategoryId == Thing.categoryPFOOD) 
						   && item.DistanceToCreature <= disMenor){
							disMenor=item.DistanceToCreature;
							menor=item;
						}
					}
					if(menor!=null){
						if(disMenor<=30.0){
							worldServer.SendEatIt(creatureId,menor.Name);
						}
						}
                        break;
                    //case ClarionAgentActionType.DO_NOTHING:
					case ClarionAgentActionType.MOVE_FOOD:
						menor=null;
						disMenor=Double.MaxValue;
						foreach (Thing item in coisas) {
							if((item.CategoryId == Thing.CATEGORY_FOOD || item.CategoryId == Thing.CATEGORY_NPFOOD || item.CategoryId == Thing.categoryPFOOD) 
						   	&& item.DistanceToCreature <= disMenor){
								disMenor=item.DistanceToCreature;
								menor=item;
						}
						}
						if(menor!=null){                    
							worldServer.SendSetGoTo(creatureId,1.0,1.0,menor.X1,menor.Y1);
						}
                        break;
				case ClarionAgentActionType.MOVE_JEWEL:
					Creature creature = null;
					foreach (Thing item in coisas) {
						if (item.CategoryId == Thing.CATEGORY_CREATURE) {
							creature = (Creature)item;
						}
					}
					if (creature != null && creature.cumpriuLeaflets()) {
						int numLeaflets=creature.NumberOfLeaflets;
						Objetivo obj=null;
						int recompensa=int.MinValue;
						int posicaoMaiorRecompensa=0;
						for (int i = 0; i < numLeaflets; i++){
							obj=creature.Objetivos[i];
							if(!obj.objetivoCumprido() && recompensa<obj.recompensa){
								recompensa=obj.recompensa;
								posicaoMaiorRecompensa=i;
							}
						}
						obj=creature.Objetivos[posicaoMaiorRecompensa];
						List<String> procurados=new List<string>();
						foreach (Leaflet leaflet in obj.leaflets) {
							if(leaflet.continuarProcurando()){
								procurados.Add(leaflet.Color);
							}
						}
						List<Thing> jewelNoMundo=new List<Thing>();
						foreach (Thing coisa in coisas) {
							if(coisa.CategoryId==Thing.CATEGORY_JEWEL && procurados.Contains(coisa.Material.Color)){
								jewelNoMundo.Add(coisa);
							}
						}
						if(jewelNoMundo.Count>0){
							Thing jewelMP=jewelNoMundo[0];
							foreach (Thing joia in jewelNoMundo) {
								if(jewelMP.DistanceToCreature >= joia.DistanceToCreature){
									jewelMP=joia;
								}
							}
							worldServer.SendSetGoTo(creatureId,1.0,1.0,jewelMP.X1,jewelMP.Y1);
						}else{
							buscaJewelMaisProxima();
						}
					} else {
						buscaJewelMaisProxima();
					}
					break;
                    case ClarionAgentActionType.ROTATE_CLOCKWISE:
					    double newprad = prad + Math.PI/1000.0;
					    while (newprad > Math.PI) newprad -= 2 * Math.PI;
			            while (newprad < - Math.PI) newprad += 2 * Math.PI;
                        worldServer.SendSetAngle(creatureId, 0, 0, newprad);
						//worldServer.SendSetAngle(creatureId, 0, 1, 0.785);    
                        break;
                    case ClarionAgentActionType.ROTATE:                    				
						double ang = prad + Math.PI/1000.0;
						while (ang > Math.PI) ang -= 2 * Math.PI;
						while (ang < - Math.PI) ang += 2 * Math.PI;
						worldServer.SendSetAngle(creatureId, 0, 0, ang);
						//angulo=angulo+0.33;
						//if(angulo>=6.28){angulo=0.0;}
					    //worldServer.SendSetAngle(creatureId, 0.0, 1.0, angulo);
					//worldServer.SendSetTurn(creatureId,0.0,2.0,0.2);
                        break;
                    default:
                        break;
                }
            }
        }

        #endregion
	}

	
}
